@@ -1,6 +1,6 @@
 package UI::Dialog::Screen::Menu;
 ###############################################################################
-#  Copyright (C) 2015  Kevin C. Krinke <kevin@krinke.ca>
+#  Copyright (C) 2004-2016  Kevin C. Krinke <kevin@krinke.ca>
 #
 #  This library is free software; you can redistribute it and/or
 #  modify it under the terms of the GNU Lesser General Public
@@ -16,14 +16,14 @@ package UI::Dialog::Screen::Menu;
 #  License along with this library; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 ###############################################################################
+use 5.006;
 use strict;
 use warnings;
-use diagnostics;
 use constant { TRUE => 1, FALSE => 0 };
 
 BEGIN {
     use vars qw($VERSION);
-    $VERSION = '1.13';
+    $VERSION = '1.14';
 }
 
 use UI::Dialog;
